@@ -501,8 +501,16 @@ print_pkenc_list( struct kidlist_item *list, int failed )
 	    }
 	}
 	else if (list->reason)
+          {
 	    log_info(_("public key decryption failed: %s\n"),
 						g10_errstr(list->reason));
+	    if (is_status_enabled ())
+              {
+		char buf[20+30];
+		snprintf (buf, sizeof buf, "pkdecrypt_failed %d", list->reason);
+		write_status_text (STATUS_ERROR, buf);
+              }
+          }
     }
 }
 
