@@ -1,11 +1,11 @@
 /* photoid.c - photo ID handling code
- * Copyright (C) 2001, 2002, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2005, 2006, 2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -41,6 +39,7 @@
 #include "main.h"
 #include "photoid.h"
 #include "ttyio.h"
+#include "trustdb.h"
 
 /* Generate a new photo id packet, or return NULL if canceled */
 PKT_user_id *
@@ -160,7 +159,7 @@ generate_photo_id(PKT_public_key *pk,const char *photo_name)
          "user" may not be able to dismiss a viewer window! */
       if(opt.command_fd==-1)
 	{
-	  show_photos(uid->attribs,uid->numattribs,pk,NULL);
+	  show_photos(uid->attribs,uid->numattribs,pk,NULL,uid);
 	  switch(cpr_get_answer_yes_no_quit("photoid.jpeg.okay",
 					 _("Is this photo correct (y/N/q)? ")))
 	    {
@@ -195,7 +194,8 @@ generate_photo_id(PKT_public_key *pk,const char *photo_name)
 }
 
 /* Returns 0 for error, 1 for valid */
-int parse_image_header(const struct user_attribute *attr,byte *type,u32 *len)
+int 
+parse_image_header(const struct user_attribute *attr,byte *type,u32 *len)
 {
   u16 headerlen;
 
@@ -230,7 +230,8 @@ int parse_image_header(const struct user_attribute *attr,byte *type,u32 *len)
    make sure it is not too big (see parse-packet.c:parse_attribute).
    Extensions should be 3 characters long for the best cross-platform
    compatibility. */
-char *image_type_to_string(byte type,int style)
+char *
+image_type_to_string(byte type,int style)
 {
   char *string;
 
@@ -291,8 +292,10 @@ get_default_photo_command(void)
 }
 #endif
 
-void show_photos(const struct user_attribute *attrs,
-		 int count,PKT_public_key *pk,PKT_secret_key *sk)
+void
+show_photos(const struct user_attribute *attrs,
+	    int count,PKT_public_key *pk,PKT_secret_key *sk,
+	    PKT_user_id *uid)
 {
 #ifndef DISABLE_PHOTO_VIEWER
   int i;
@@ -303,6 +306,8 @@ void show_photos(const struct user_attribute *attrs,
   memset(&args,0,sizeof(args));
   args.pk=pk;
   args.sk=sk;
+  args.validity_info=get_validity_info(pk,uid);
+  args.validity_string=get_validity_string(pk,uid);
 
   if(pk)
     keyid_from_pk(pk,kid);
