@@ -1,12 +1,12 @@
 /* sign.c - sign data
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007, 2010 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -69,32 +67,17 @@ mk_notation_policy_etc( PKT_signature *sig,
     struct notation *nd=NULL;
     struct expando_args args;
 
+    assert(sig->version>=4);
+
     memset(&args,0,sizeof(args));
     args.pk=pk;
     args.sk=sk;
 
-    /* It is actually impossible to get here when making a v3 key
-       signature since keyedit.c:sign_uids will automatically bump a
-       signature with a notation or policy url up to v4, but it is
-       good to do these checks anyway. */
-
     /* notation data */
     if(IS_SIG(sig) && opt.sig_notations)
-      {
-	if(sig->version<4)
-	  log_error(_("can't put notation data into v3 (PGP 2.x style) "
-		      "signatures\n"));
-	else
-	  nd=opt.sig_notations;
-      }
+      nd=opt.sig_notations;
     else if( IS_CERT(sig) && opt.cert_notations )
-      {
-	if(sig->version<4)
-	  log_error(_("can't put notation data into v3 (PGP 2.x style) "
-		      "key signatures\n"));
-	else
-	  nd=opt.cert_notations;
-      }
+      nd=opt.cert_notations;
 
     if(nd)
       {
@@ -119,21 +102,9 @@ mk_notation_policy_etc( PKT_signature *sig,
 
     /* set policy URL */
     if( IS_SIG(sig) && opt.sig_policy_url )
-      {
-	if(sig->version<4)
-	  log_error(_("can't put a policy URL into v3 (PGP 2.x style) "
-		      "signatures\n"));
-	else
-	  pu=opt.sig_policy_url;
-      }
+      pu=opt.sig_policy_url;
     else if( IS_CERT(sig) && opt.cert_policy_url )
-      {
-	if(sig->version<4)
-	  log_error(_("can't put a policy URL into v3 key (PGP 2.x style) "
-		      "signatures\n"));
-	else
-	  pu=opt.cert_policy_url;
-      }
+      pu=opt.cert_policy_url;
 
     for(;pu;pu=pu->next)
       {
@@ -156,12 +127,7 @@ mk_notation_policy_etc( PKT_signature *sig,
 
     /* preferred keyserver URL */
     if( IS_SIG(sig) && opt.sig_keyserver_url )
-      {
-	if(sig->version<4)
-	  log_info("can't put a preferred keyserver URL into v3 signatures\n");
-	else
-	  pu=opt.sig_keyserver_url;
-      }
+      pu=opt.sig_keyserver_url;
 
     for(;pu;pu=pu->next)
       {
@@ -308,8 +274,7 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
         xfree (snbuf);
         if (!rc)
           {
-            sig->data[0] = mpi_alloc ( (rbuflen+BYTES_PER_MPI_LIMB-1)
-                                       / BYTES_PER_MPI_LIMB );
+            sig->data[0] = mpi_alloc ( mpi_nlimb_hint_from_nbytes (rbuflen) );
             mpi_set_buffer (sig->data[0], rbuf, rbuflen, 0);
             xfree (rbuf);
           }
@@ -319,28 +284,6 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
       }
     else 
       {
-#if 0
-	/* Disabled for now.  It seems reasonable to accept a
-	   truncated hash for a DSA1 key, even though we don't
-	   generate it without --enable-dsa2.  Be liberal in what you
-	   accept, etc. */
-
-	/* If it's a DSA key, and q is 160 bits, it might be an
-	   old-style DSA key.  If the hash doesn't match the q, fail
-	   unless --enable-dsa2 is set.  If the q isn't 160 bits, then
-	   allow any hash since it must be a DSA2 key (if the hash is
-	   too small, we'll fail in encode_md_value). */
-
-	if(sk->pubkey_algo==PUBKEY_ALGO_DSA
-	   && (mpi_get_nbits(sk->skey[1])/8)==20
-	   && !opt.flags.dsa2
-	   && md_digest_length(digest_algo)!=20)
-	  {
-	    log_error(_("DSA requires the use of a 160 bit hash algorithm\n"));
-	    return G10ERR_GENERAL;
-	  }
-#endif
-
         frame = encode_md_value( NULL, sk, md, digest_algo );
         if (!frame)
           return G10ERR_GENERAL;
@@ -471,12 +414,15 @@ hash_for(PKT_secret_key *sk)
 
       return match_dsa_hash(qbytes);
     }
-  else if(sk->is_protected && sk->protect.s2k.mode==1002)
+  else if (sk->is_protected && sk->protect.s2k.mode == 1002
+           && sk->protect.ivlen == 16
+           && !memcmp (sk->protect.iv, "\xD2\x76\x00\x01\x24\x01\x01", 7))
     {
-      /* The sk lives on a smartcard, and current smartcards only
-	 handle SHA-1 and RIPEMD/160.  This is correct now, but may
-	 need revision as the cards add algorithms. */
-
+      /* The sk lives on a smartcard, and old smartcards only handle
+	 SHA-1 and RIPEMD/160.  Newer smartcards (v2.0) don't have
+	 this restriction anymore.  Fortunately the serial number
+	 encodes the version of the card and thus we know that this
+	 key is on a v1 card. */
       if(opt.personal_digest_prefs)
 	{
 	  prefitem_t *prefs;
@@ -595,7 +541,8 @@ write_onepass_sig_packets (SK_LIST sk_list, IOBUF out, int sigclass )
  * Helper to write the plaintext (literal data) packet
  */
 static int
-write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
+write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname,
+			int ptmode, u32 timestamp)
 {
     PKT_plaintext *pt = NULL;
     u32 filesize;
@@ -611,7 +558,7 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
         int overflow;
 
         if( !(tmpsize = iobuf_get_filelength(inp, &overflow))
-            && !overflow )
+            && !overflow && opt.verbose)
 	  log_info (_("WARNING: `%s' is an empty file\n"), fname);
 
         /* We can't encode the length of very large files because
@@ -636,7 +583,7 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
     if (!opt.no_literal) {
         PACKET pkt;
 
-        pt->timestamp = make_timestamp ();
+        pt->timestamp = timestamp;
         pt->mode = ptmode;
         pt->len = filesize;
         pt->new_ctb = !pt->len && !RFC1991;
@@ -711,8 +658,10 @@ write_signature_packets (SK_LIST sk_list, IOBUF out, MD_HANDLE hash,
 	md = md_copy (hash);
 
 	if (sig->version >= 4)
+	  {
 	    build_sig_subpkt_from_sig (sig);
-	mk_notation_policy_etc (sig, NULL, sk);
+	    mk_notation_policy_etc (sig, NULL, sk);
+	  }
 
         hash_sigversion_to_magic (md, sig);
 	md_final (md);
@@ -772,7 +721,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     SK_LIST sk_list = NULL;
     SK_LIST sk_rover = NULL;
     int multifile = 0;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -797,9 +746,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
@@ -866,8 +815,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if (DBG_HASHING)
 	md_start_debug (mfx.md, "sign");
 
-   /* If we're encrypting and signing, it is reasonable to pick the
-       hash algorithm to use out of the recepient key prefs. */
+    /* If we're encrypting and signing, it is reasonable to pick the
+       hash algorithm to use out of the recepient key prefs.  This is
+       best effort only, as in a DSA2 and smartcard world there are
+       cases where we cannot please everyone with a single hash (DSA2
+       wants >160 and smartcards want =160).  In the future this could
+       be more complex with different hashes for each sk, but the
+       current design requires a single hash for all SKs. */
     if(pk_list)
       {
 	if(opt.def_digest_algo)
@@ -883,43 +837,56 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	  }
 	else
 	  {
-	    int hashlen=0,algo;
+	    union pref_hint hint;
+	    int algo,smartcard=0;
+
+	    hint.digest_length=0;
 
 	    /* Of course, if the recipient asks for something
-	       unreasonable (like a non-160-bit hash for DSA without
-	       --enable-dsa2, for example), then don't do it.  Check
-	       all sk's - if any are DSA, then the hash has
-	       restrictions.  In the future this can be more complex
-	       with different hashes for each sk, but so long as there
-	       is only one signing algorithm with hash restrictions,
-	       this is ok. -dms */
+	       unreasonable (like the wrong hash for a DSA key) then
+	       don't do it.  Check all sk's - if any are DSA or live
+	       on a smartcard, then the hash has restrictions and we
+	       may not be able to give the recipient what they want.
+	       For DSA, pass a hint for the largest q we have.  Note
+	       that this means that a q>160 key will override a q=160
+	       key and force the use of truncation for the q=160 key.
+	       The alternative would be to ignore the recipient prefs
+	       completely and get a different hash for each DSA key in
+	       hash_for().  The override behavior here is more or less
+	       reasonable as it is under the control of the user which
+	       keys they sign with for a given message and the fact
+	       that the message with multiple signatures won't be
+	       usable on an implementation that doesn't understand
+	       DSA2 anyway. */
 
 	    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next )
 	      {
 		if(sk_rover->sk->pubkey_algo==PUBKEY_ALGO_DSA)
 		  {
-		    if(opt.flags.dsa2)
-		      hashlen=mpi_get_nbits(sk_rover->sk->skey[1])/8;
-		    else
-		      hashlen=20;
-		    break;
+		    int temp_hashlen=mpi_get_nbits(sk_rover->sk->skey[1])/8;
+
+		    /* Pick a hash that is large enough for our
+		       largest q */
+
+		    if(hint.digest_length<temp_hashlen)
+		      hint.digest_length=temp_hashlen;
 		  }
 		else if(sk_rover->sk->is_protected
 			&& sk_rover->sk->protect.s2k.mode==1002)
-		  {
-		    /* Current smartcards only do 160-bit hashes.
-		       Note that this may well have to change as the
-		       cards add algorithms. */
-
-		    hashlen=20;
-		    break;
-		  }
+		  smartcard=1;
 	      }
 
-	    if((algo=
-		select_algo_from_prefs(pk_list,PREFTYPE_HASH,-1,
-				       hashlen?&hashlen:NULL))>0)
-	      recipient_digest_algo=algo;
+	    /* Current smartcards only do 160-bit hashes.  If we have
+	       to have a >160-bit hash, then we can't use the
+	       recipient prefs as we'd need both =160 and >160 at the
+	       same time and recipient prefs currently require a
+	       single hash for all signatures.  All this may well have
+	       to change as the cards add algorithms. */
+
+	    if(!smartcard || (smartcard && hint.digest_length==20))
+	      if((algo=
+		  select_algo_from_prefs(pk_list,PREFTYPE_HASH,-1,&hint))>0)
+		recipient_digest_algo=algo;
 	  }
       }
 
@@ -1031,7 +998,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     }
     else {
         rc = write_plaintext_packet (out, inp, fname,
-                                     opt.textmode && !outfile ? 't':'b');
+                                     opt.textmode && !outfile ? 't':'b',
+				     create_time);
     }
 
     /* catch errors from above */
@@ -1041,7 +1009,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     /* write the signatures */
     rc = write_signature_packets (sk_list, out, mfx.md,
                                   opt.textmode && !outfile? 0x01 : 0x00,
-				  0, duration, detached ? 'D':'S');
+				  create_time, duration, detached ? 'D':'S');
     if( rc )
         goto leave;
 
@@ -1080,7 +1048,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     SK_LIST sk_rover = NULL;
     int old_style = RFC1991;
     int only_md5 = 0;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
@@ -1088,9 +1056,9 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
@@ -1200,7 +1168,8 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     iobuf_push_filter( out, armor_filter, &afx );
 
     /* write the signatures */
-    rc=write_signature_packets (sk_list, out, textmd, 0x01, 0, duration, 'C');
+    rc=write_signature_packets (sk_list, out, textmd, 0x01,
+				create_time, duration, 'C');
     if( rc )
         goto leave;
 
@@ -1235,7 +1204,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     SK_LIST sk_list = NULL;
     SK_LIST sk_rover = NULL;
     int algo;
-    u32 duration=0;
+    u32 create_time=make_timestamp(),duration=0;
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -1247,9 +1216,9 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     if(!opt.force_v3_sigs && !RFC1991)
       {
 	if(opt.ask_sig_expire && !opt.batch)
-	  duration=ask_expire_interval(1,opt.def_sig_expire);
+	  duration=ask_expire_interval(create_time,1,opt.def_sig_expire);
 	else
-	  duration=parse_expire_string(opt.def_sig_expire);
+	  duration=parse_expire_string(create_time,opt.def_sig_expire);
       }
 
     rc = build_sk_list (locusr, &sk_list, 1, PUBKEY_USAGE_SIG);
@@ -1353,7 +1322,8 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* Pipe data through all filters; i.e. write the signed stuff */
     /*(current filters: zip - encrypt - armor)*/
-    rc = write_plaintext_packet (out, inp, fname, opt.textmode ? 't':'b');
+    rc = write_plaintext_packet (out, inp, fname, opt.textmode ? 't':'b',
+				 create_time);
     if (rc)
 	goto leave;
     
@@ -1361,7 +1331,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     /*(current filters: zip - encrypt - armor)*/
     rc = write_signature_packets (sk_list, out, mfx.md,
 				  opt.textmode? 0x01 : 0x00,
-				  0, duration, 'S');
+				  create_time, duration, 'S');
     if( rc )
         goto leave;
 
@@ -1475,8 +1445,10 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
       sig->expiredate=sig->timestamp+duration;
     sig->sig_class = sigclass;
     if( sig->version >= 4 )
+      {
 	build_sig_subpkt_from_sig( sig );
-    mk_notation_policy_etc( sig, pk, sk );
+	mk_notation_policy_etc( sig, pk, sk );
+      }
 
     /* Crucial that the call to mksubpkt comes LAST before the calls
        to finalize the sig as that makes it possible for the mksubpkt
