@@ -1,11 +1,11 @@
 /* curl-shim.h
- * Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ * Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _CURL_SHIM_H_
@@ -50,7 +48,9 @@ typedef enum
     CURLOPT_CAINFO,
     CURLOPT_POST,
     CURLOPT_POSTFIELDS,
-    CURLOPT_FAILONERROR
+    CURLOPT_FAILONERROR,
+    CURLOPT_HTTPHEADER,
+    CURLOPT_SRVTAG_GPG_HACK
   } CURLoption;
 
 typedef size_t (*write_func)(char *buffer,size_t size,
@@ -65,8 +65,10 @@ typedef struct
   write_func writer;
   void *file;
   char *postfields;
+  char *srvtag;
   unsigned int status;
   FILE *errors;
+  struct curl_slist *headers;
   struct
   {
     unsigned int post:1;
@@ -76,8 +78,14 @@ typedef struct
   struct http_context hd;
 } CURL;
 
+typedef struct
+{
+  const char **protocols;
+} curl_version_info_data; 
+
 #define CURL_ERROR_SIZE 256
 #define CURL_GLOBAL_DEFAULT 0
+#define CURLVERSION_NOW 0
 
 CURLcode curl_global_init(long flags);
 void curl_global_cleanup(void);
@@ -86,7 +94,17 @@ CURLcode curl_easy_setopt(CURL *curl,CURLoption option,...);
 CURLcode curl_easy_perform(CURL *curl);
 void curl_easy_cleanup(CURL *curl);
 char *curl_easy_escape(CURL *curl,char *str,int len);
-void curl_free(char *ptr);
-#define curl_version() "GnuPG curl-shim "VERSION
+#define curl_free(x) free(x)
+#define curl_version() "GnuPG curl-shim"
+curl_version_info_data *curl_version_info(int type);
+
+struct curl_slist
+{
+  STRLIST list;
+};
+
+struct curl_slist *curl_slist_append(struct curl_slist *list,
+				     const char *string);
+void curl_slist_free_all(struct curl_slist *list);
 
 #endif /* !_CURL_SHIM_H_ */
