@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -52,6 +50,10 @@
 #include <readline/history.h>
 #endif
 
+#ifdef __VMS
+# include "vms.h"
+#endif /* __VMS */
+
 #include "util.h"
 #include "memory.h"
 #include "ttyio.h"
@@ -124,7 +126,31 @@ cleanup(void)
 }
 #endif
 
-static void
+void
+tty_cleanup_after_signal (void)
+{
+#ifdef HAVE_TCGETATTR
+  cleanup ();
+#endif
+}
+
+#ifdef __VMS
+/* 2006-08-10 SMS.
+   Interface function needed for VMS (unless someone
+   wishes to make "ttyfp" global).  See g10_log_print_prefix() in
+   util/logger.c.  */
+FILE *
+ttyfp_is (void)
+{
+  return ttyfp;
+}
+#endif /* def __VMS */
+
+
+#ifndef __VMS
+static
+#endif
+       void
 init_ttyfp(void)
 {
     if( initialized )
@@ -233,13 +259,14 @@ tty_printf( const char *fmt, ... )
     va_start( arg_ptr, fmt ) ;
 #ifdef _WIN32
     {   
-        char *buf = NULL;
+        char *buf;
         int n;
 	DWORD nwritten;
 
-	n = vasprintf(&buf, fmt, arg_ptr);
-	if( !buf )
-	    log_bug("vasprintf() failed\n");
+	buf = xtryvasprintf(fmt, arg_ptr);
+	if (!buf)
+          log_bug("xtryvasprintf() failed\n");
+        n = strlen (buf);
         
 	if (!WriteConsoleA (con.out, buf, n, &nwritten, NULL))
 	    log_fatal ("WriteConsole failed: %s", w32_strerror (0));
@@ -280,13 +307,14 @@ tty_fprintf (FILE *fp, const char *fmt, ... )
     va_start( arg_ptr, fmt ) ;
 #ifdef _WIN32
     {   
-        char *buf = NULL;
+        char *buf;
         int n;
 	DWORD nwritten;
 
-	n = vasprintf(&buf, fmt, arg_ptr);
-	if( !buf )
-	    log_bug("vasprintf() failed\n");
+	buf = xtryvasprintf (fmt, arg_ptr);
+	if (!buf)
+          log_bug ("xtryvasprintf() failed\n");
+        n = strlen (buf);
         
 	if (!WriteConsoleA (con.out, buf, n, &nwritten, NULL))
 	    log_fatal ("WriteConsole failed: %s", w32_strerror (0));
@@ -405,7 +433,8 @@ do_get( const char *prompt, int hidden )
 	init_ttyfp();
 
     last_prompt_len = 0;
-    buf = xmalloc(n=50);
+    n = 50;
+    buf = hidden ? xmalloc_secure(n) : xmalloc(n);
     i = 0;
 
 #ifdef _WIN32 /* windoze version */
@@ -498,6 +527,11 @@ do_get( const char *prompt, int hidden )
 	if( tcsetattr( fileno(ttyfp), TCSAFLUSH, &term ) )
 	    log_fatal("tcsetattr() failed: %s\n", strerror(errno) );
 #endif
+# ifdef __VMS
+        /* Disable terminal echo. */
+        if (vms_set_term_echo (0))
+          log_fatal ("error disabling terminal echo: %s\n", strerror (errno));
+# endif /* __VMS */
     }
 
     tty_printf( "%s", prompt );
@@ -530,11 +564,16 @@ do_get( const char *prompt, int hidden )
 
 
     if( hidden ) {
-#ifdef HAVE_TCGETATTR
+# ifdef HAVE_TCGETATTR
 	if( tcsetattr(fileno(ttyfp), TCSAFLUSH, &termsave) )
 	    log_error("tcsetattr() failed: %s\n", strerror(errno) );
 	restore_termios = 0;
-#endif
+# endif
+# ifdef __VMS
+        /* Restore (most likely enable) terminal echo. */
+        if (vms_set_term_echo( -1))
+          log_fatal ("error enabling terminal echo: %s\n", strerror (errno));
+# endif /* __VMS */
     }
 #endif /* end unix version */
     buf[i] = 0;
